MODULE PVM_Annex
  ! Laurent Gasser (lga@sma.ch) 
  ! Swiss Meteorological Institute, Zuerich
  ! January 31, 97
  ! Fortran 90 calls to PVM 3.3.11 for predefined reduction operators

IMPLICIT NONE

  ! PROGRAMMER'S NOTE: The datatype argument cannot be remove, since
  ! the routines are internally called.  It would be more elegant
  ! to rely on the argument type and completely avoid the datatype
  ! argument.  The same holds for the count.

  ! PROGRAMMER'S NOTE: The generic name should not be the same as 
  ! the FORTRAN 77 inner call.  If both are the same, we define a
  ! recursive function for each of the instance of PVM_min,...
  INTERFACE pvm_min
     MODULE PROCEDURE PvmMin_shortinteger
     MODULE PROCEDURE PvmMin_shortinteger_array
     MODULE PROCEDURE PvmMin_integer
     MODULE PROCEDURE PvmMin_integer_array
     MODULE PROCEDURE PvmMin_longinteger
     MODULE PROCEDURE PvmMin_longinteger_array
     MODULE PROCEDURE PvmMin_real
     MODULE PROCEDURE PvmMin_real_array
     MODULE PROCEDURE PvmMin_longreal
     MODULE PROCEDURE PvmMin_longreal_array
     MODULE PROCEDURE PvmMin_complex
     MODULE PROCEDURE PvmMin_complex_array
     MODULE PROCEDURE PvmMin_longcomplex
     MODULE PROCEDURE PvmMin_longcomplex_array
  END INTERFACE

  INTERFACE pvm_max
     MODULE PROCEDURE PvmMax_shortinteger
     MODULE PROCEDURE PvmMax_shortinteger_array
     MODULE PROCEDURE PvmMax_integer
     MODULE PROCEDURE PvmMax_integer_array
     MODULE PROCEDURE PvmMax_longinteger
     MODULE PROCEDURE PvmMax_longinteger_array
     MODULE PROCEDURE PvmMax_real
     MODULE PROCEDURE PvmMax_real_array
     MODULE PROCEDURE PvmMax_longreal
     MODULE PROCEDURE PvmMax_longreal_array
     MODULE PROCEDURE PvmMax_complex
     MODULE PROCEDURE PvmMax_complex_array
     MODULE PROCEDURE PvmMax_longcomplex
     MODULE PROCEDURE PvmMax_longcomplex_array
  END INTERFACE

  INTERFACE pvm_sum
     MODULE PROCEDURE PvmSum_integer
     MODULE PROCEDURE PvmSum_integer_array
     MODULE PROCEDURE PvmSum_longinteger
     MODULE PROCEDURE PvmSum_longinteger_array
     MODULE PROCEDURE PvmSum_real
     MODULE PROCEDURE PvmSum_real_array
     MODULE PROCEDURE PvmSum_longreal
     MODULE PROCEDURE PvmSum_longreal_array
     MODULE PROCEDURE PvmSum_complex
     MODULE PROCEDURE PvmSum_complex_array
     MODULE PROCEDURE PvmSum_longcomplex
     MODULE PROCEDURE PvmSum_longcomplex_array
  END INTERFACE

  INTERFACE pvm_product
     MODULE PROCEDURE PvmProduct_integer
     MODULE PROCEDURE PvmProduct_integer_array
     MODULE PROCEDURE PvmProduct_longinteger
     MODULE PROCEDURE PvmProduct_longinteger_array
     MODULE PROCEDURE PvmProduct_real
     MODULE PROCEDURE PvmProduct_real_array
     MODULE PROCEDURE PvmProduct_longreal
     MODULE PROCEDURE PvmProduct_longreal_array
     MODULE PROCEDURE PvmProduct_complex
     MODULE PROCEDURE PvmProduct_complex_array
     MODULE PROCEDURE PvmProduct_longcomplex
     MODULE PROCEDURE PvmProduct_longcomplex_array
  END INTERFACE

CONTAINS

  SUBROUTINE PvmMin_shortinteger (datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=1), INTENT(INOUT) :: minimum
    INTEGER(KIND=1), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_shortinteger

  SUBROUTINE PvmMin_shortinteger_array (datatype, minimum, new_comer, &
                                        count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=1), INTENT(INOUT) :: minimum(:)
    INTEGER(KIND=1), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_shortinteger_array

  SUBROUTINE PvmMin_integer (datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: minimum
    INTEGER(KIND=2), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info

    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_integer

  SUBROUTINE PvmMin_integer_array(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: minimum(:)
    INTEGER(KIND=2), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info

    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_integer_array

  SUBROUTINE PvmMin_longinteger(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: minimum
    INTEGER(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info

    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_longinteger

  SUBROUTINE PvmMin_longinteger_array(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: minimum(:)
    INTEGER(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_longinteger_array

  SUBROUTINE PvmMin_real(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: minimum
    REAL(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_real

  SUBROUTINE PvmMin_real_array(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: minimum(:)
    REAL(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_real_array

  SUBROUTINE PvmMin_longreal(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: minimum
    REAL(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_longreal

  SUBROUTINE PvmMin_longreal_array(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: minimum(:)
    REAL(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_longreal_array

  SUBROUTINE PvmMin_complex(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: minimum
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_complex

  SUBROUTINE PvmMin_complex_array(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: minimum(:)
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_complex_array

  SUBROUTINE PvmMin_longcomplex(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: minimum
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_longcomplex

  SUBROUTINE PvmMin_longcomplex_array(datatype, minimum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: minimum(:)
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMin(datatype, minimum, new_comer, count, info)
  END SUBROUTINE PvmMin_longcomplex_array

  SUBROUTINE PvmMax_shortinteger (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=1), INTENT(INOUT) :: maximum
    INTEGER(KIND=1), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_shortinteger

  SUBROUTINE PvmMax_shortinteger_array (datatype, maximum, new_comer, &
                                        count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=1), INTENT(INOUT) :: maximum(:)
    INTEGER(KIND=1), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_shortinteger_array

  SUBROUTINE PvmMax_integer (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: maximum
    INTEGER(KIND=2), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_integer

  SUBROUTINE PvmMax_integer_array (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: maximum(:)
    INTEGER(KIND=2), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_integer_array

  SUBROUTINE PvmMax_longinteger (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: maximum
    INTEGER(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_longinteger

  SUBROUTINE PvmMax_longinteger_array (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: maximum(:)
    INTEGER(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_longinteger_array

  SUBROUTINE PvmMax_real (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: maximum
    REAL(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_real

  SUBROUTINE PvmMax_real_array (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: maximum(:)
    REAL(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_real_array

  SUBROUTINE PvmMax_longreal (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: maximum
    REAL(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_longreal

  SUBROUTINE PvmMax_longreal_array (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: maximum(:)
    REAL(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_longreal_array

  SUBROUTINE PvmMax_complex (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: maximum
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_complex

  SUBROUTINE PvmMax_complex_array (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: maximum(:)
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_complex_array

  SUBROUTINE PvmMax_longcomplex (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: maximum
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_longcomplex

  SUBROUTINE PvmMax_longcomplex_array (datatype, maximum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: maximum(:)
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmMax(datatype, maximum, new_comer, count, info)
  END SUBROUTINE PvmMax_longcomplex_array

  SUBROUTINE PvmSum_integer (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: sum
    INTEGER(KIND=2), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_integer

  SUBROUTINE PvmSum_integer_array (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: sum(:)
    INTEGER(KIND=2), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_integer_array

  SUBROUTINE PvmSum_longinteger (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: sum
    INTEGER(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_longinteger

  SUBROUTINE PvmSum_longinteger_array (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: sum(:)
    INTEGER(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_longinteger_array

  SUBROUTINE PvmSum_real (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: sum
    REAL(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_real

  SUBROUTINE PvmSum_real_array (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: sum(:)
    REAL(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_real_array

  SUBROUTINE PvmSum_longreal (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: sum
    REAL(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_longreal

  SUBROUTINE PvmSum_longreal_array (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: sum(:)
    REAL(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_longreal_array

  SUBROUTINE PvmSum_complex (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: sum
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_complex

  SUBROUTINE PvmSum_complex_array (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: sum(:)
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_complex_array

  SUBROUTINE PvmSum_longcomplex (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: sum
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_longcomplex

  SUBROUTINE PvmSum_longcomplex_array (datatype, sum, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: sum(:)
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmSum(datatype, sum, new_comer, count, info)
  END SUBROUTINE PvmSum_longcomplex_array

  SUBROUTINE PvmProduct_integer (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: product
    INTEGER(KIND=2), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_integer

  SUBROUTINE PvmProduct_integer_array (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=2), INTENT(INOUT) :: product(:)
    INTEGER(KIND=2), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_integer_array

  SUBROUTINE PvmProduct_longinteger (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: product
    INTEGER(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_longinteger

  SUBROUTINE PvmProduct_longinteger_array (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    INTEGER(KIND=4), INTENT(INOUT) :: product(:)
    INTEGER(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_longinteger_array

  SUBROUTINE PvmProduct_real (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: product
    REAL(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_real

  SUBROUTINE PvmProduct_real_array (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=4), INTENT(INOUT) :: product(:)
    REAL(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_real_array

  SUBROUTINE PvmProduct_longreal (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: product
    REAL(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_longreal

  SUBROUTINE PvmProduct_longreal_array (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)         :: datatype
    REAL(KIND=8), INTENT(INOUT) :: product(:)
    REAL(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(OUT)        :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_longreal_array

  SUBROUTINE PvmProduct_complex (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: product
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_complex

  SUBROUTINE PvmProduct_complex_array (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=4), INTENT(INOUT) :: product(:)
    COMPLEX(KIND=4), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_complex_array

  SUBROUTINE PvmProduct_longcomplex (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: product
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_longcomplex

  SUBROUTINE PvmProduct_longcomplex_array (datatype, product, new_comer, count, info)
    IMPLICIT NONE

    INTEGER, INTENT(IN)            :: datatype
    COMPLEX(KIND=8), INTENT(INOUT) :: product(:)
    COMPLEX(KIND=8), INTENT(IN)    :: new_comer(:)
    INTEGER, INTENT(IN)            :: count
    INTEGER, INTENT(OUT)           :: info
    
    CALL PvmProduct(datatype, product, new_comer, count, info)
  END SUBROUTINE PvmProduct_longcomplex_array

END MODULE PVM_Annex
