MODULE PVM_Interfaces
  ! Laurent Gasser (lga@sma.ch) 
  ! Swiss Meteorological Institute, Zuerich
  ! 15 October 1996
  ! Fortran 90 calls to PVM 3.3.11

  ! January 31, 97: Laurent Gasser
  ! Rework the interface for pvm_reduce, by giving a true interface
  ! to the function argument.  Define headers for the predefined 
  ! reduction operators PvmMin, PvmMax, PvmSum, and PvmProduct.
  ! These do not fully belong to the PVM_Interfaces, and are put
  ! in the PVM_Annex module.

  ! Every time it is possible, remove the descriptor of data type and 
  ! array length. Fortran 90 is able to identify them.

  ! Identifiers cannot exceed 31 character in Fortran 90.  Consequently
  ! root is sometimes abbriged as r.

  ! Overloadings are defined on the five basic types defined by Fortran 90:
  ! INTEGER, REAL, COMPLEX, LOGICAL, CHARACTER

  ! on Sun, Solaris 2.4 with CRAY f90 compiler, three kinds of integer:
  !  The bits size of integer(1) is 8
  !  The bits size of integer(2) is 16
  !  The bits size of integer(4) is 32
  !  The bits size of integer(8) is 32
  !  The bits size of integer is 32
  
  ! two kinds of real:
  !  The number of binary digits of real(4) is 24
  !  The number of binary digits of real(8) is 53
  !  The number of binary digits of real(16) is 53
  !  The number of binary digits of real is 24

  ! two kinds of complex:
  !  The number of decimal digits of complex(4) is 6
  !  The number of decimal digits of complex(8) is 15
  !  The number of decimal digits of complex(16) is 15
  !  The number of decimal digits of complex is 6

  ! one kind of logical:
  !  .TRUE.   = $00000001
  !  .FALSE.  = $00000000

  ! one kind of character:
  !  character = $FF000000 for ACHAR(255) (packed array of bytes)

  ! OBSOLETE
  ! pvm_advise, pvm_serror

  USE PVM_Types
  USE PVM_Constants
  USE PVM_Annex

  IMPLICIT NONE

  INTERFACE pvm_addhost
     SUBROUTINE pvmfaddhost(host, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: host
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmfaddhost
  END INTERFACE

  INTERFACE pvm_archcode
     SUBROUTINE pvmfarchode(arch, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: arch
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmfarchode
  END INTERFACE

  INTERFACE pvm_barrier
     SUBROUTINE pvmfbarrier(group, count, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(IN)      :: count
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmfbarrier
  END INTERFACE

  INTERFACE pvm_bcast
     SUBROUTINE pvmfbcast(group, msgtag, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(IN)      :: msgtag
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmfbcast
  END INTERFACE

  INTERFACE pvm_bufinfo
     SUBROUTINE pvmfbufinfo(bufid, bytes, msgtag, tid, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: bufid
       INTEGER, INTENT(OUT) :: bytes
       INTEGER, INTENT(OUT) :: msgtag
       INTEGER, INTENT(OUT) :: tid
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfbufinfo
  END INTERFACE

  INTERFACE pvm_catchout
     MODULE PROCEDURE pvm_boolean_catchout
  END INTERFACE

  INTERFACE pvm_config
     MODULE PROCEDURE pvm_typed_config
  END INTERFACE

  INTERFACE pvm_delhost
     SUBROUTINE pvmfdelhost(host, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: host
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmfdelhost
  END INTERFACE

  INTERFACE pvm_exit
     SUBROUTINE pvmfexit(info)
       IMPLICIT NONE
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfexit
  END INTERFACE

  INTERFACE pvm_freebuf
     SUBROUTINE pvmffreebuf(bufid, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: bufid
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmffreebuf
  END INTERFACE

  INTERFACE pvm_freezegroup
     SUBROUTINE pvmffreezegroup(group, size, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(IN)      :: size
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmffreezegroup
  END INTERFACE

  INTERFACE pvm_gather
     MODULE PROCEDURE pvm_gather_root_shortinteger
     MODULE PROCEDURE pvm_gather_r_shortint_array
     MODULE PROCEDURE pvm_gather_shortinteger
     MODULE PROCEDURE pvm_gather_shortinteger_array
     MODULE PROCEDURE pvm_gather_root_integer
     MODULE PROCEDURE pvm_gather_root_integer_array
     MODULE PROCEDURE pvm_gather_integer
     MODULE PROCEDURE pvm_gather_integer_array
     MODULE PROCEDURE pvm_gather_root_longinteger
     MODULE PROCEDURE pvm_gather_r_longint_array
     MODULE PROCEDURE pvm_gather_longinteger
     MODULE PROCEDURE pvm_gather_longinteger_array
     MODULE PROCEDURE pvm_gather_root_real
     MODULE PROCEDURE pvm_gather_root_real_array
     MODULE PROCEDURE pvm_gather_real
     MODULE PROCEDURE pvm_gather_real_array
     MODULE PROCEDURE pvm_gather_root_longreal
     MODULE PROCEDURE pvm_gather_root_longreal_array
     MODULE PROCEDURE pvm_gather_longreal
     MODULE PROCEDURE pvm_gather_longreal_array
     MODULE PROCEDURE pvm_gather_root_complex
     MODULE PROCEDURE pvm_gather_root_complex_array
     MODULE PROCEDURE pvm_gather_complex
     MODULE PROCEDURE pvm_gather_complex_array
     MODULE PROCEDURE pvm_gather_root_longcomplex
     MODULE PROCEDURE pvm_gather_r_longcmplx_array
     MODULE PROCEDURE pvm_gather_longcomplex
     MODULE PROCEDURE pvm_gather_longcomplex_array
     MODULE PROCEDURE pvm_gather_root_logical
     MODULE PROCEDURE pvm_gather_root_logical_array
     MODULE PROCEDURE pvm_gather_logical
     MODULE PROCEDURE pvm_gather_logical_array
     MODULE PROCEDURE pvm_gather_root_string
     MODULE PROCEDURE pvm_gather_string
     MODULE PROCEDURE pvm_gather_root_character_array
     MODULE PROCEDURE pvm_gather_character_array
  END INTERFACE

  INTERFACE pvm_getinst
     SUBROUTINE pvmfgetinst(group, tid, inum)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(IN)      :: tid
       INTEGER, INTENT(OUT)     :: inum
     END SUBROUTINE pvmfgetinst
  END INTERFACE

  INTERFACE pvm_getopt
     SUBROUTINE pvmfgetopt(what, val)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: what
       INTEGER, INTENT(OUT) :: val
     END SUBROUTINE pvmfgetopt
  END INTERFACE

  INTERFACE pvm_getrbuf
     SUBROUTINE pvmfgetrbuf(bufid)
       IMPLICIT NONE
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmfgetrbuf
  END INTERFACE

  INTERFACE pvm_getsbuf
     SUBROUTINE pvmfgetsbuf(bufid)
       IMPLICIT NONE
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmfgetsbuf
  END INTERFACE

  INTERFACE pvm_gettid
     SUBROUTINE pvmfgettid(group,inum,tid)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(IN)      :: inum
       INTEGER, INTENT(OUT)     :: tid
     END SUBROUTINE pvmfgettid
  END INTERFACE

  INTERFACE pvm_gsize
     SUBROUTINE pvmfgsize(group, size)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(OUT)     :: size
     END SUBROUTINE pvmfgsize
  END INTERFACE

  INTERFACE pvm_halt
     SUBROUTINE pvmfhalt(info)
       IMPLICIT NONE
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfhalt
  END INTERFACE

  INTERFACE pvm_hostsync
     SUBROUTINE pvmfhostsync(host, clksec, clkusec, deltasec, deltausec, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: host
       INTEGER, INTENT(OUT) :: clksec
       INTEGER, INTENT(OUT) :: clkusec
       INTEGER, INTENT(OUT) :: deltasec
       INTEGER, INTENT(OUT) :: deltausec
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfhostsync
  END INTERFACE

  INTERFACE pvm_initsend
     SUBROUTINE pvmfinitsend(encoding, bufid)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: encoding
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmfinitsend
  END INTERFACE

  INTERFACE pvm_joingroup
     SUBROUTINE pvmfjoingroup(group,inum)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(OUT)     :: inum
     END SUBROUTINE pvmfjoingroup
  END INTERFACE

  INTERFACE pvm_kill
     SUBROUTINE pvmfkill(tid, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfkill
  END INTERFACE

  INTERFACE pvm_lvgroup
     SUBROUTINE pvmflvgroup(group, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: group
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmflvgroup
  END INTERFACE

  INTERFACE pvm_mcast
     ! Beware, ntask and tids are swapped compared to C headers.
     SUBROUTINE pvmfmcast(ntask, tids, msgtag, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: ntask
       INTEGER, INTENT(IN)  :: tids(ntask)
       INTEGER, INTENT(IN)  :: msgtag
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfmcast
  END INTERFACE

  INTERFACE pvm_mkbuf
     SUBROUTINE pvmfmkbuf(encoding, bufid)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: encoding
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmfmkbuf
  END INTERFACE

  INTERFACE pvm_mstat
     SUBROUTINE pvmfmstat(host, mstat)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: host
       INTEGER, INTENT(OUT)     :: mstat
     END SUBROUTINE pvmfmstat
  END INTERFACE

  INTERFACE pvm_mytid
     SUBROUTINE pvmfmytid(tid)
       IMPLICIT NONE
       INTEGER, INTENT(OUT) :: tid
     END SUBROUTINE pvmfmytid
  END INTERFACE

  INTERFACE pvm_notify
     SUBROUTINE pvmfnotify(what, msgtag, count, tids, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: what
       INTEGER, INTENT(IN)  :: msgtag
       INTEGER, INTENT(IN)  :: count
       INTEGER, INTENT(IN)  :: tids(:)
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfnotify
  END INTERFACE

  INTERFACE pvm_nrecv
     SUBROUTINE pvmfnrecv(tid, msgtag, bufid)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(IN)  :: msgtag
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmfnrecv
  END INTERFACE

  INTERFACE pvm_pack
     MODULE PROCEDURE pvm_pack_shortinteger
     MODULE PROCEDURE pvm_pack_shortinteger_array
     MODULE PROCEDURE pvm_pack_integer
     MODULE PROCEDURE pvm_pack_integer_array
     MODULE PROCEDURE pvm_pack_longinteger
     MODULE PROCEDURE pvm_pack_longinteger_array
     MODULE PROCEDURE pvm_pack_real
     MODULE PROCEDURE pvm_pack_real_array
     MODULE PROCEDURE pvm_pack_longreal
     MODULE PROCEDURE pvm_pack_longreal_array
     MODULE PROCEDURE pvm_pack_complex
     MODULE PROCEDURE pvm_pack_complex_array
     MODULE PROCEDURE pvm_pack_longcomplex
     MODULE PROCEDURE pvm_pack_longcomplex_array
     MODULE PROCEDURE pvm_pack_logical
     MODULE PROCEDURE pvm_pack_logical_array
     MODULE PROCEDURE pvm_pack_string
     MODULE PROCEDURE pvm_pack_character_array
  END INTERFACE

  INTERFACE pvm_parent
     SUBROUTINE pvmfparent(tid)
       IMPLICIT NONE
       INTEGER, INTENT(OUT) :: tid
     END SUBROUTINE pvmfparent
  END INTERFACE

  INTERFACE pvm_perror
     SUBROUTINE pvmfperror(msg, info)
       IMPLICIT NONE
       CHARACTER(*), INTENT(IN) :: msg
       INTEGER, INTENT(OUT)     :: info
     END SUBROUTINE pvmfperror 
  END INTERFACE

  INTERFACE pvm_precv
     MODULE PROCEDURE pvm_precv_shortinteger
     MODULE PROCEDURE pvm_precv_shortinteger_array
     MODULE PROCEDURE pvm_precv_integer
     MODULE PROCEDURE pvm_precv_integer_array
     MODULE PROCEDURE pvm_precv_longinteger
     MODULE PROCEDURE pvm_precv_longinteger_array
     MODULE PROCEDURE pvm_precv_real
     MODULE PROCEDURE pvm_precv_real_array
     MODULE PROCEDURE pvm_precv_longreal
     MODULE PROCEDURE pvm_precv_longreal_array
     MODULE PROCEDURE pvm_precv_complex
     MODULE PROCEDURE pvm_precv_complex_array
     MODULE PROCEDURE pvm_precv_longcomplex
     MODULE PROCEDURE pvm_precv_longcomplex_array
     MODULE PROCEDURE pvm_precv_logical
     MODULE PROCEDURE pvm_precv_logical_array
     MODULE PROCEDURE pvm_precv_string
     MODULE PROCEDURE pvm_precv_character_array
  END INTERFACE

  INTERFACE pvm_probe
     SUBROUTINE pvmfprobe(tid, msgtag, bufid)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(IN)  :: msgtag
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmfprobe
  END INTERFACE

  INTERFACE pvm_psend
     MODULE PROCEDURE pvm_psend_shortinteger
     MODULE PROCEDURE pvm_psend_shortinteger_array
     MODULE PROCEDURE pvm_psend_integer
     MODULE PROCEDURE pvm_psend_integer_array
     MODULE PROCEDURE pvm_psend_longinteger
     MODULE PROCEDURE pvm_psend_longinteger_array
     MODULE PROCEDURE pvm_psend_real
     MODULE PROCEDURE pvm_psend_real_array
     MODULE PROCEDURE pvm_psend_longreal
     MODULE PROCEDURE pvm_psend_longreal_array
     MODULE PROCEDURE pvm_psend_complex
     MODULE PROCEDURE pvm_psend_complex_array
     MODULE PROCEDURE pvm_psend_longcomplex
     MODULE PROCEDURE pvm_psend_longcomplex_array
     MODULE PROCEDURE pvm_psend_logical
     MODULE PROCEDURE pvm_psend_logical_array
     MODULE PROCEDURE pvm_psend_string
     MODULE PROCEDURE pvm_psend_character_array
  END INTERFACE

  INTERFACE pvm_pstat
     SUBROUTINE pvmfpstat(tid, status)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(OUT) :: status
     END SUBROUTINE pvmfpstat
  END INTERFACE

  INTERFACE pvm_recv
     SUBROUTINE pvmfrecv(tid, msgtag, bufid)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(IN)  :: msgtag
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmfrecv
  END INTERFACE

  INTERFACE pvm_reduce
     MODULE PROCEDURE pvm_reduce_shortinteger
     MODULE PROCEDURE pvm_reduce_shortinteger_array
     MODULE PROCEDURE pvm_reduce_integer
     MODULE PROCEDURE pvm_reduce_integer_array
     MODULE PROCEDURE pvm_reduce_longinteger
     MODULE PROCEDURE pvm_reduce_longinteger_array
     MODULE PROCEDURE pvm_reduce_real
     MODULE PROCEDURE pvm_reduce_real_array
     MODULE PROCEDURE pvm_reduce_longreal
     MODULE PROCEDURE pvm_reduce_longreal_array
     MODULE PROCEDURE pvm_reduce_complex
     MODULE PROCEDURE pvm_reduce_complex_array
     MODULE PROCEDURE pvm_reduce_longcomplex
     MODULE PROCEDURE pvm_reduce_longcomplex_array
     MODULE PROCEDURE pvm_reduce_logical
     MODULE PROCEDURE pvm_reduce_logical_array
     MODULE PROCEDURE pvm_reduce_string
     MODULE PROCEDURE pvm_reduce_character_array
  END INTERFACE

  INTERFACE pvm_scatter
     MODULE PROCEDURE pvm_scatter_root_shortinteger
     MODULE PROCEDURE pvm_scatter_r_shortint_array
     MODULE PROCEDURE pvm_scatter_shortinteger
     MODULE PROCEDURE pvm_scatter_shortinteger_array
     MODULE PROCEDURE pvm_scatter_root_integer
     MODULE PROCEDURE pvm_scatter_root_integer_array
     MODULE PROCEDURE pvm_scatter_integer
     MODULE PROCEDURE pvm_scatter_integer_array
     MODULE PROCEDURE pvm_scatter_root_longinteger
     MODULE PROCEDURE pvm_scatter_r_longint_array
     MODULE PROCEDURE pvm_scatter_longinteger
     MODULE PROCEDURE pvm_scatter_longinteger_array
     MODULE PROCEDURE pvm_scatter_root_real
     MODULE PROCEDURE pvm_scatter_root_real_array
     MODULE PROCEDURE pvm_scatter_real
     MODULE PROCEDURE pvm_scatter_real_array
     MODULE PROCEDURE pvm_scatter_root_longreal
     MODULE PROCEDURE pvm_scatter_root_longreal_array
     MODULE PROCEDURE pvm_scatter_longreal
     MODULE PROCEDURE pvm_scatter_longreal_array
     MODULE PROCEDURE pvm_scatter_root_complex
     MODULE PROCEDURE pvm_scatter_root_complex_array
     MODULE PROCEDURE pvm_scatter_complex
     MODULE PROCEDURE pvm_scatter_complex_array
     MODULE PROCEDURE pvm_scatter_root_longcomplex
     MODULE PROCEDURE pvm_scatter_r_longcmplx_array
     MODULE PROCEDURE pvm_scatter_longcomplex
     MODULE PROCEDURE pvm_scatter_longcomplex_array
     MODULE PROCEDURE pvm_scatter_root_logical
     MODULE PROCEDURE pvm_scatter_root_logical_array
     MODULE PROCEDURE pvm_scatter_logical
     MODULE PROCEDURE pvm_scatter_logical_array
     MODULE PROCEDURE pvm_scatter_root_string
     MODULE PROCEDURE pvm_scatter_string
     MODULE PROCEDURE pvm_scatter_r_character_array
     MODULE PROCEDURE pvm_scatter_character_array
  END INTERFACE

  INTERFACE pvm_send
     SUBROUTINE pvmfsend(tid, msgtag, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: msgtag
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfsend
  END INTERFACE

  INTERFACE pvm_sendsig
     SUBROUTINE pvmfsendsig(tid, signum, info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(IN)  :: signum
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfsendsig
  END INTERFACE

  INTERFACE pvm_setopt
     SUBROUTINE pvmfsetopt(what, val, oldval)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: what
       INTEGER, INTENT(IN)  :: val
       INTEGER, INTENT(OUT) :: oldval
     END SUBROUTINE pvmfsetopt
  END INTERFACE

  INTERFACE pvm_setrbuf
     SUBROUTINE pvmfsetrbuf(bufid, oldbuf)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: bufid
       INTEGER, INTENT(OUT) :: oldbuf
     END SUBROUTINE pvmfsetrbuf
  END INTERFACE

  INTERFACE pvm_setsbuf
     SUBROUTINE pvmfsetsbuf(bufid, oldbuf)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: bufid
       INTEGER, INTENT(OUT) :: oldbuf
     END SUBROUTINE pvmfsetsbuf
  END INTERFACE

  INTERFACE pvm_spawn
     MODULE PROCEDURE pvm_spawn_one
     MODULE PROCEDURE pvm_spawn_many
  END INTERFACE

  INTERFACE pvm_start_pvmd
     SUBROUTINE pvmfstartpvmd(args, block,info)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: args
       INTEGER, INTENT(IN)  :: block ! ought to be logical
       INTEGER, INTENT(OUT) :: info
     END SUBROUTINE pvmfstartpvmd
  END INTERFACE

  INTERFACE pvm_tasks
     MODULE PROCEDURE pvm_typed_tasks
  END INTERFACE

  INTERFACE pvm_tidtohost
     SUBROUTINE pvmftidtohost(tid, dtid)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(OUT) :: dtid
     END SUBROUTINE pvmftidtohost
  END INTERFACE

  INTERFACE pvm_trecv
     SUBROUTINE pvmftrecv(tid, msgtag, sec, usec, bufid)
       IMPLICIT NONE
       INTEGER, INTENT(IN)  :: tid
       INTEGER, INTENT(IN)  :: msgtag
       INTEGER, INTENT(IN)  :: sec
       INTEGER, INTENT(IN)  :: usec
       INTEGER, INTENT(OUT) :: bufid
     END SUBROUTINE pvmftrecv
  END INTERFACE

  INTERFACE pvm_unpack
     MODULE PROCEDURE pvm_unpack_shortinteger
     MODULE PROCEDURE pvm_unpack_shortinteger_array
     MODULE PROCEDURE pvm_unpack_integer
     MODULE PROCEDURE pvm_unpack_integer_array
     MODULE PROCEDURE pvm_unpack_longinteger
     MODULE PROCEDURE pvm_unpack_longinteger_array
     MODULE PROCEDURE pvm_unpack_real
     MODULE PROCEDURE pvm_unpack_real_array
     MODULE PROCEDURE pvm_unpack_longreal
     MODULE PROCEDURE pvm_unpack_longreal_array
     MODULE PROCEDURE pvm_unpack_complex
     MODULE PROCEDURE pvm_unpack_complex_array
     MODULE PROCEDURE pvm_unpack_longcomplex
     MODULE PROCEDURE pvm_unpack_longcomplex_array
     MODULE PROCEDURE pvm_unpack_logical
     MODULE PROCEDURE pvm_unpack_logical_array
     MODULE PROCEDURE pvm_unpack_string
     MODULE PROCEDURE pvm_unpack_character_array
  END INTERFACE
  

CONTAINS

  
  SUBROUTINE pvm_boolean_catchout(active, info)
    IMPLICIT NONE
    LOGICAL, INTENT(IN)  :: active
    INTEGER, INTENT(OUT) :: info
    IF (ACTIVE) THEN
       CALL pvmfcatchout(1, info)
    ELSE
       CALL pvmfcatchout(0, info)
    END IF
  END SUBROUTINE pvm_boolean_catchout

  SUBROUTINE pvm_typed_config(nhost, narch, host_info, info)
    IMPLICIT NONE
    INTEGER, INTENT(OUT)           :: nhost
    INTEGER, INTENT(OUT)           :: narch
    TYPE(pvmhostinfo), INTENT(OUT) :: host_info
    INTEGER, INTENT(OUT)           :: info

    CALL pvmfconfig(nhost, narch, host_info%hi_tid, host_info%hi_name, &
                    host_info%hi_arch, host_info%hi_speed, info)
  END SUBROUTINE pvm_typed_config


  SUBROUTINE pvm_gather_root_shortinteger(result, data, msgtag, group, &
                                          rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(OUT) :: result(:)
    INTEGER(KIND=1), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! BYTE1 == 1
    CALL pvmfgather(result, data, 1, 1, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_shortinteger

  SUBROUTINE pvm_gather_r_shortint_array(result, data, msgtag, group, &
                                         rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(OUT) :: result(:)
    INTEGER(KIND=1), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! BYTE1 == 1
    CALL pvmfgather(result, data, size(data,1), 1, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_r_shortint_array

  SUBROUTINE pvm_gather_shortinteger(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! BYTE1 == 1
    CALL pvmfgather(0, data, 1, 1, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_shortinteger

  SUBROUTINE pvm_gather_shortinteger_array(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! BYTE1 == 1
    CALL pvmfgather(0, data, size(data,1), 1, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_shortinteger_array

  SUBROUTINE pvm_gather_root_integer(result, data, msgtag, group, &
                                     rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(OUT) :: result(:)
    INTEGER(KIND=2), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER2 == 2
    CALL pvmfgather(result, data, 1, 2, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_integer

  SUBROUTINE pvm_gather_root_integer_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(OUT) :: result(:)
    INTEGER(KIND=2), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER2 == 2
    CALL pvmfgather(result, data, size(data,1), 2, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_root_integer_array

  SUBROUTINE pvm_gather_integer(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER2 == 2
    CALL pvmfgather(0, data, 1, 2, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_integer

  SUBROUTINE pvm_gather_integer_array(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER2 == 2
    CALL pvmfgather(0, data, size(data,1), 2, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_integer_array

  SUBROUTINE pvm_gather_root_longinteger(result, data, msgtag, group, &
                                         rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(OUT) :: result(:)
    INTEGER(KIND=4), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER4 == 3
    CALL pvmfgather(result, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_longinteger

  SUBROUTINE pvm_gather_r_longint_array(result, data, msgtag, group, &
                                        rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(OUT) :: result(:)
    INTEGER(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER4 == 3
    CALL pvmfgather(result, data, size(data,1), 3, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_r_longint_array

  SUBROUTINE pvm_gather_longinteger(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfgather(0, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_longinteger

  SUBROUTINE pvm_gather_longinteger_array(data, msgtag, group, &
                                          rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfgather(0, data, size(data,1), 3, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_longinteger_array

  SUBROUTINE pvm_gather_root_real(result, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(OUT) :: result(:)
    REAL(KIND=4), INTENT(IN)  :: data
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL4 == 4
    CALL pvmfgather(result, data, 1, 4, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_real

  SUBROUTINE pvm_gather_root_real_array(result, data, msgtag, group, &
                                        rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(OUT) :: result(:)
    REAL(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL4 == 4
    CALL pvmfgather(result, data, size(data,1), 4, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_root_real_array

  SUBROUTINE pvm_gather_real(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(IN)  :: data
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL4 == 4
    CALL pvmfgather(0, data, 1, 4, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_real

  SUBROUTINE pvm_gather_real_array(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL4 == 4
    CALL pvmfgather(0, data, size(data,1), 4, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_real_array

  SUBROUTINE pvm_gather_root_longreal(result, data, msgtag, group, &
                                      rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(OUT) :: result(:)
    REAL(KIND=8), INTENT(IN)  :: data
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL8 == 6
    CALL pvmfgather(result, data, 1, 6, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_longreal

  SUBROUTINE pvm_gather_root_longreal_array(result, data, msgtag, group, &
                                            rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(OUT) :: result(:)
    REAL(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL8 == 6
    CALL pvmfgather(result, data, size(data,1), 6, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_root_longreal_array

  SUBROUTINE pvm_gather_longreal(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN)  :: data
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL8 == 6
    CALL pvmfgather(0, data, 1, 6, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_longreal

  SUBROUTINE pvm_gather_longreal_array(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL8 == 6
    CALL pvmfgather(0, data, size(data,1), 6, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_longreal_array

  SUBROUTINE pvm_gather_root_complex(result, data, msgtag, group, &
                                     rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(OUT) :: result(:)
    COMPLEX(KIND=4), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX8 = 5
    CALL pvmfgather(result, data, 1, 5, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_complex

  SUBROUTINE pvm_gather_root_complex_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(OUT) :: result(:)
    COMPLEX(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX8 = 5
    CALL pvmfgather(result, data, size(data,1), 5, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_root_complex_array

  SUBROUTINE pvm_gather_complex(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX8 = 5
    CALL pvmfgather(0, data, 1, 5, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_complex

  SUBROUTINE pvm_gather_complex_array(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX8 = 5
    CALL pvmfgather(0, data, size(data,1), 5, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_complex_array

  SUBROUTINE pvm_gather_root_longcomplex(result, data, msgtag, group, &
                                         rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(OUT) :: result(:)
    COMPLEX(KIND=8), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX16= 7
    CALL pvmfgather(result, data, 1, 7, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_longcomplex

  SUBROUTINE pvm_gather_r_longcmplx_array(result, data, msgtag, group, &
                                          rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(OUT) :: result(:)
    COMPLEX(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX16= 7
    CALL pvmfgather(result, data, size(data,1), 7, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_r_longcmplx_array

  SUBROUTINE pvm_gather_longcomplex(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(IN)  :: data
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX16= 7
    CALL pvmfgather(0, data, 1, 7, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_longcomplex

  SUBROUTINE pvm_gather_longcomplex_array(data, msgtag, group, &
                                          rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX16= 7
    CALL pvmfgather(0, data, size(data,1), 7, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_longcomplex_array

  SUBROUTINE pvm_gather_root_logical(result, data, msgtag, group, &
                                     rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT)     :: result(:)
    LOGICAL, INTENT(IN)      :: data
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! INTEGER4 == 3
    CALL pvmfgather(result, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_root_logical

  SUBROUTINE pvm_gather_root_logical_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT)     :: result(:)
    LOGICAL, INTENT(IN)      :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! INTEGER4 == 3
    CALL pvmfgather(result, data, size(data,1), 3, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_root_logical_array

  SUBROUTINE pvm_gather_logical(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(IN)      :: data
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfgather(0, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_logical

  SUBROUTINE pvm_gather_logical_array(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(IN)      :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfgather(0, data, size(data,1), 3, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_logical_array

  SUBROUTINE pvm_gather_root_string(result, data, msgtag, group, &
                                    rootginst, info)
    IMPLICIT NONE
    CHARACTER(*), INTENT(OUT) :: result
    CHARACTER(*), INTENT(IN)  :: data
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! STRING == 0
    CALL pvmfgather(result, data, len(data), 0, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_root_string

  SUBROUTINE pvm_gather_string(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    CHARACTER(*), INTENT(IN)  :: data
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! STRING == 0
    CALL pvmfgather(0, data, len(data), 0, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_gather_string

  SUBROUTINE pvm_gather_root_character_array(result, data, msgtag, group, &
                                             rootginst, info)
    IMPLICIT NONE
    CHARACTER, INTENT(OUT)   :: result(:)
    CHARACTER, INTENT(IN)    :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! STRING == 0
    CALL pvmfgather(result, data, size(data,1), 0, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_root_character_array

  SUBROUTINE pvm_gather_character_array(data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    CHARACTER, INTENT(IN)    :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    INTEGER(KIND=1) :: ignored
    ! STRING == 0
    CALL pvmfgather(0, data, size(data,1), 0, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_gather_character_array


  SUBROUTINE pvm_typed_tasks(where, ntask, task_info, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)            :: where
    INTEGER, INTENT(OUT)           :: ntask
    TYPE(pvmtaskinfo), INTENT(OUT) :: task_info
    INTEGER, INTENT(OUT)           :: info

    CALL pvmftasks(where, ntask, task_info%ti_tid, task_info%ti_ptid, &
                   task_info%ti_host, task_info%ti_flag, task_info%ti_a_out,&
                   info)
  END SUBROUTINE pvm_typed_tasks

  SUBROUTINE pvm_pack_shortinteger(data, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(1, data, 1, 1, info) ! 1 == BYTE1
  END SUBROUTINE pvm_pack_shortinteger

  SUBROUTINE pvm_pack_shortinteger_array(data, count, stride, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(1, data, count, stride, info) ! 1 == BYTE1
  END SUBROUTINE pvm_pack_shortinteger_array

  SUBROUTINE pvm_pack_integer(data, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(2, data, 1, 1, info) ! 2 == INTEGER2
  END SUBROUTINE pvm_pack_integer

  SUBROUTINE pvm_pack_integer_array(data, count, stride, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(2, data, count, stride, info) ! 2 == INTEGER2
  END SUBROUTINE pvm_pack_integer_array

  SUBROUTINE pvm_pack_longinteger(data, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(3, data, 1, 1, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_pack_longinteger

  SUBROUTINE pvm_pack_longinteger_array(data, count, stride, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(3, data, count, stride, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_pack_longinteger_array

  SUBROUTINE pvm_pack_real(data, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(IN) :: data
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfpack(4, data, 1, 1, info) ! 4 == REAL4
  END SUBROUTINE pvm_pack_real

  SUBROUTINE pvm_pack_real_array(data, count, stride, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)      :: count
    INTEGER, INTENT(IN)      :: stride
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfpack(4, data, count, stride, info) ! 4 == REAL4
  END SUBROUTINE pvm_pack_real_array

  SUBROUTINE pvm_pack_longreal(data, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN) :: data
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfpack(6, data, 1, 1, info) ! 6 == REAL8
  END SUBROUTINE pvm_pack_longreal

  SUBROUTINE pvm_pack_longreal_array(data, count, stride, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)      :: count
    INTEGER, INTENT(IN)      :: stride
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfpack(6, data, count, stride, info) ! 6 == REAL8
  END SUBROUTINE pvm_pack_longreal_array

  SUBROUTINE pvm_pack_complex(data, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(5, data, 1, 1, info) ! 5 == COMPLEX8
  END SUBROUTINE pvm_pack_complex

  SUBROUTINE pvm_pack_complex_array(data, count, stride, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(5, data, count, stride, info) ! 5 == COMPLEX8
  END SUBROUTINE pvm_pack_complex_array

  SUBROUTINE pvm_pack_longcomplex(data, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(7, data, 1, 1, info) ! 7 == COMPLEX16
  END SUBROUTINE pvm_pack_longcomplex

  SUBROUTINE pvm_pack_longcomplex_array(data, count, stride, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfpack(7, data, count, stride, info) ! 7 == COMPLEX16
  END SUBROUTINE pvm_pack_longcomplex_array

  SUBROUTINE pvm_pack_logical(data, info)
    IMPLICIT NONE
    LOGICAL, INTENT(IN)  :: data
    INTEGER, INTENT(OUT) :: info

    CALL pvmfpack(3, data, 1, 1, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_pack_logical

  SUBROUTINE pvm_pack_logical_array(data, count, stride, info)
    IMPLICIT NONE
    LOGICAL, INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)  :: count
    INTEGER, INTENT(IN)  :: stride
    INTEGER, INTENT(OUT) :: info

    CALL pvmfpack(3, data, count, stride, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_pack_logical_array

  SUBROUTINE pvm_pack_string(data, info)
    IMPLICIT NONE
    CHARACTER(*), INTENT(IN) :: data
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfpack(0, data, LEN(data), 1, info) ! 0 == STRING
  END SUBROUTINE pvm_pack_string

  SUBROUTINE pvm_pack_character_array(data, count, stride, info)
    IMPLICIT NONE
    CHARACTER, INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)   :: count
    INTEGER, INTENT(IN)   :: stride
    INTEGER, INTENT(OUT)  :: info

    CALL pvmfpack(0, data, SIZE(data, 1), 1, info) ! 0 == STRING
  END SUBROUTINE pvm_pack_character_array


  SUBROUTINE pvm_precv_shortinteger(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    INTEGER(KIND=1), INTENT(OUT) :: buf
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 1 == BYTE1
    CALL pvmfprecv(tid, msgtag, buf, 1, 1, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_shortinteger

  SUBROUTINE pvm_precv_shortinteger_array(tid, msgtag, buf,  atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    INTEGER(KIND=1), INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 1 == BYTE1
    CALL pvmfprecv(tid, msgtag, buf, size(buf,1), 1, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_shortinteger_array

  SUBROUTINE pvm_precv_integer(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    INTEGER(KIND=2), INTENT(OUT) :: buf 
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 2 == INTEGER2
    CALL pvmfprecv(tid, msgtag, buf, 1, 2, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_integer

  SUBROUTINE pvm_precv_integer_array(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    INTEGER(KIND=2), INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 2 == INTEGER2
    CALL pvmfprecv(tid, msgtag, buf, size(buf,1), 2, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_integer_array

  SUBROUTINE pvm_precv_longinteger(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    INTEGER(KIND=4), INTENT(OUT) :: buf
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 3 == INTEGER4
    CALL pvmfprecv(tid, msgtag, buf, 1, 3, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_longinteger

  SUBROUTINE pvm_precv_longinteger_array(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    INTEGER(KIND=4), INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 3 == INTEGER4
    CALL pvmfprecv(tid, msgtag, buf, size(buf, 1), 3, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_longinteger_array

  SUBROUTINE pvm_precv_real(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)       :: tid
    INTEGER, INTENT(IN)       :: msgtag
    REAL(KIND=4), INTENT(OUT) :: buf
    INTEGER, INTENT(OUT)      :: atid
    INTEGER, INTENT(OUT)      :: atag
    INTEGER, INTENT(OUT)      :: alen
    INTEGER, INTENT(OUT)      :: info

     ! 4 == REAL4
    CALL pvmfprecv(tid, msgtag, buf, 1, 4, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_real

  SUBROUTINE pvm_precv_real_array(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)       :: tid
    INTEGER, INTENT(IN)       :: msgtag
    REAL(KIND=4), INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)      :: atid
    INTEGER, INTENT(OUT)      :: atag
    INTEGER, INTENT(OUT)      :: alen
    INTEGER, INTENT(OUT)      :: info

     ! 4 == REAL4
    CALL pvmfprecv(tid, msgtag, buf, size(buf, 1), 4, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_real_array

  SUBROUTINE pvm_precv_longreal(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)       :: tid
    INTEGER, INTENT(IN)       :: msgtag
    REAL(KIND=8), INTENT(OUT) :: buf
    INTEGER, INTENT(OUT)      :: atid
    INTEGER, INTENT(OUT)      :: atag
    INTEGER, INTENT(OUT)      :: alen
    INTEGER, INTENT(OUT)      :: info

     ! 6 == REAL8
    CALL pvmfprecv(tid, msgtag, buf, 1, 6, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_longreal

  SUBROUTINE pvm_precv_longreal_array(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)       :: tid
    INTEGER, INTENT(IN)       :: msgtag
    REAL(KIND=8), INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)      :: atid
    INTEGER, INTENT(OUT)      :: atag
    INTEGER, INTENT(OUT)      :: alen
    INTEGER, INTENT(OUT)      :: info

     ! 6 == REAL8
    CALL pvmfprecv(tid, msgtag, buf, size(buf, 1), 6, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_longreal_array

  SUBROUTINE pvm_precv_complex(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    COMPLEX(KIND=4), INTENT(OUT) :: buf
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 5 == COMPLEX8
    CALL pvmfprecv(tid, msgtag, buf, 1, 5, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_complex

  SUBROUTINE pvm_precv_complex_array(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    COMPLEX(KIND=4), INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 5 == COMPLEX8
    CALL pvmfprecv(tid, msgtag, buf, size(buf, 1), 5, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_complex_array

  SUBROUTINE pvm_precv_longcomplex(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    COMPLEX(KIND=8), INTENT(OUT) :: buf
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 7 == COMPLEX16
    CALL pvmfprecv(tid, msgtag, buf, 1, 7, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_longcomplex

  SUBROUTINE pvm_precv_longcomplex_array(tid, msgtag, buf, atid, atag, alen, &
                                         info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)          :: tid
    INTEGER, INTENT(IN)          :: msgtag
    COMPLEX(KIND=8), INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)         :: atid
    INTEGER, INTENT(OUT)         :: atag
    INTEGER, INTENT(OUT)         :: alen
    INTEGER, INTENT(OUT)         :: info

     ! 7 == COMPLEX16
    CALL pvmfprecv(tid, msgtag, buf, size(buf, 1), 7, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_longcomplex_array

  SUBROUTINE pvm_precv_logical(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)  :: tid
    INTEGER, INTENT(IN)  :: msgtag
    LOGICAL, INTENT(OUT) :: buf
    INTEGER, INTENT(OUT) :: atid
    INTEGER, INTENT(OUT) :: atag
    INTEGER, INTENT(OUT) :: alen
    INTEGER, INTENT(OUT) :: info

    ! 3 == INTEGER4
    CALL pvmfprecv(tid, msgtag, buf, 1, 3, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_logical

  SUBROUTINE pvm_precv_logical_array(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)  :: tid
    INTEGER, INTENT(IN)  :: msgtag
    LOGICAL, INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT) :: atid
    INTEGER, INTENT(OUT) :: atag
    INTEGER, INTENT(OUT) :: alen
    INTEGER, INTENT(OUT) :: info

    ! 3 == INTEGER4
    CALL pvmfprecv(tid, msgtag, buf, size(buf, 1), 3, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_logical_array

  SUBROUTINE pvm_precv_string(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)       :: tid
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(OUT) :: buf
    INTEGER, INTENT(OUT)      :: atid
    INTEGER, INTENT(OUT)      :: atag
    INTEGER, INTENT(OUT)      :: alen
    INTEGER, INTENT(OUT)      :: info

    ! 0 == STRING
    CALL pvmfprecv(tid, msgtag, buf, len(buf), 0, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_string

  SUBROUTINE pvm_precv_character_array(tid, msgtag, buf, atid, atag, alen, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)    :: tid
    INTEGER, INTENT(IN)    :: msgtag
    CHARACTER, INTENT(OUT) :: buf(:)
    INTEGER, INTENT(OUT)   :: atid
    INTEGER, INTENT(OUT)   :: atag
    INTEGER, INTENT(OUT)   :: alen
    INTEGER, INTENT(OUT)   :: info

    ! 0 == STRING
    CALL pvmfprecv(tid, msgtag, buf, size(buf, 1), 0, atid, atag, alen, info)
  END SUBROUTINE pvm_precv_character_array


  SUBROUTINE pvm_psend_shortinteger(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    INTEGER(KIND=1), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)        :: info

    ! 1 == BYTE1
    CALL pvmfpsend(tid, msgtag, buf, 1, 1, info)
  END SUBROUTINE pvm_psend_shortinteger

  SUBROUTINE pvm_psend_shortinteger_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    INTEGER(KIND=1), INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)        :: info

    ! 1 == BYTE1
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 1, info)
  END SUBROUTINE pvm_psend_shortinteger_array

  SUBROUTINE pvm_psend_integer(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    INTEGER(KIND=2), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)        :: info

    ! 2 == INTEGER2
    CALL pvmfpsend(tid, msgtag, buf, 1, 2, info)
  END SUBROUTINE pvm_psend_integer

  SUBROUTINE pvm_psend_integer_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    INTEGER(KIND=2), INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)        :: info

    ! 2 == INTEGER2
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 2, info)
  END SUBROUTINE pvm_psend_integer_array

  SUBROUTINE pvm_psend_longinteger(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    INTEGER(KIND=4), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)        :: info

    ! 3 == INTEGER4
    CALL pvmfpsend(tid, msgtag, buf, 1, 3, info)
  END SUBROUTINE pvm_psend_longinteger

  SUBROUTINE pvm_psend_longinteger_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    INTEGER(KIND=4), INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)        :: info

    ! 3 == INTEGER4
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 3, info)
  END SUBROUTINE pvm_psend_longinteger_array

  SUBROUTINE pvm_psend_real(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)      :: tid
    INTEGER, INTENT(IN)      :: msgtag
    REAL(KIND=4), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)     :: info

    ! 4 == REAL4
    CALL pvmfpsend(tid, msgtag, buf, 1, 4, info)
  END SUBROUTINE pvm_psend_real

  SUBROUTINE pvm_psend_real_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)      :: tid
    INTEGER, INTENT(IN)      :: msgtag
    REAL(KIND=4), INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)     :: info

    ! 4 == REAL4
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 4, info)
  END SUBROUTINE pvm_psend_real_array

  SUBROUTINE pvm_psend_longreal(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)      :: tid
    INTEGER, INTENT(IN)      :: msgtag
    REAL(KIND=8), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)     :: info

    ! 6 == REAL8
    CALL pvmfpsend(tid, msgtag, buf, 1, 6, info)
  END SUBROUTINE pvm_psend_longreal

  SUBROUTINE pvm_psend_longreal_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)      :: tid
    INTEGER, INTENT(IN)      :: msgtag
    REAL(KIND=8), INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)     :: info

    ! 6 == REAL8
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 6, info)
  END SUBROUTINE pvm_psend_longreal_array

  SUBROUTINE pvm_psend_complex(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    COMPLEX(KIND=4), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)        :: info

    ! 5 == COMPLEX8
    CALL pvmfpsend(tid, msgtag, buf, 1, 5, info)
  END SUBROUTINE pvm_psend_complex

  SUBROUTINE pvm_psend_complex_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    COMPLEX(KIND=4), INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)        :: info

    ! 5 == COMPLEX8
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 5, info)
  END SUBROUTINE pvm_psend_complex_array

  SUBROUTINE pvm_psend_longcomplex(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    COMPLEX(KIND=8), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)        :: info

    ! 7 == COMPLEX16
    CALL pvmfpsend(tid, msgtag, buf, 1, 7, info)
  END SUBROUTINE pvm_psend_longcomplex

  SUBROUTINE pvm_psend_longcomplex_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)         :: tid
    INTEGER, INTENT(IN)         :: msgtag
    COMPLEX(KIND=8), INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)        :: info

    ! 7 == COMPLEX16
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 7, info)
  END SUBROUTINE pvm_psend_longcomplex_array

  SUBROUTINE pvm_psend_logical(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)  :: tid
    INTEGER, INTENT(IN)  :: msgtag
    LOGICAL, INTENT(IN)  :: buf
    INTEGER, INTENT(OUT) :: info

    ! 3 == INTEGER4
    CALL pvmfpsend(tid, msgtag, buf, 1, 3, info)
  END SUBROUTINE pvm_psend_logical

  SUBROUTINE pvm_psend_logical_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)  :: tid
    INTEGER, INTENT(IN)  :: msgtag
    LOGICAL, INTENT(IN)  :: buf(:)
    INTEGER, INTENT(OUT) :: info

    ! 3 == INTEGER4
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 3, info)
  END SUBROUTINE pvm_psend_logical_array

  SUBROUTINE pvm_psend_string(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)      :: tid
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: buf
    INTEGER, INTENT(OUT)     :: info

    ! 0 == STRING
    CALL pvmfpsend(tid, msgtag, buf, len(buf), 0, info)
  END SUBROUTINE pvm_psend_string

  SUBROUTINE pvm_psend_character_array(tid, msgtag, buf, info)
    IMPLICIT NONE
    INTEGER, INTENT(IN)   :: tid
    INTEGER, INTENT(IN)   :: msgtag
    CHARACTER, INTENT(IN) :: buf(:)
    INTEGER, INTENT(OUT)  :: info

    ! 0 == STRING
    CALL pvmfpsend(tid, msgtag, buf, size(buf, 1), 0, info)
  END SUBROUTINE pvm_psend_character_array


  SUBROUTINE pvm_reduce_shortinteger(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         INTEGER(KIND=1), INTENT(INOUT) :: reduction
         INTEGER(KIND=1), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    INTEGER(KIND=1), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 1 == BYTE1
    CALL pvmfreduce(func, data, 1, 1, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_shortinteger

  SUBROUTINE pvm_reduce_shortinteger_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         INTEGER(KIND=1), INTENT(INOUT) :: reduction(:)
         INTEGER(KIND=1), INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    INTEGER(KIND=1), INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 1 == BYTE1
    CALL pvmfreduce(func, data, size(data, 1), 1, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_shortinteger_array

  SUBROUTINE pvm_reduce_integer(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         INTEGER(KIND=2), INTENT(INOUT) :: reduction
         INTEGER(KIND=2), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    INTEGER(KIND=2), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 2 == INTEGER2
    CALL pvmfreduce(func, data, 1, 2, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_integer

  SUBROUTINE pvm_reduce_integer_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         INTEGER(KIND=2), INTENT(INOUT) :: reduction(:)
         INTEGER(KIND=2), INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    INTEGER(KIND=2), INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 2 == INTEGER2
    CALL pvmfreduce(func, data, size(data, 1), 2, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_integer_array

  SUBROUTINE pvm_reduce_longinteger(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         INTEGER(KIND=4), INTENT(INOUT) :: reduction
         INTEGER(KIND=4), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    INTEGER(KIND=4), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 3 == INTEGER4
    CALL pvmfreduce(func, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_longinteger

  SUBROUTINE pvm_reduce_longinteger_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         INTEGER(KIND=4), INTENT(INOUT) :: reduction(:)
         INTEGER(KIND=4), INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    INTEGER(KIND=4), INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 3 == INTEGER4
    CALL pvmfreduce(func, data, size(data, 1), 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_longinteger_array

  SUBROUTINE pvm_reduce_real(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)         :: datatype ! redundant 
         REAL(KIND=4), INTENT(INOUT) :: reduction
         REAL(KIND=4), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)         :: count
         INTEGER, INTENT(OUT)        :: info
       END SUBROUTINE func
    END INTERFACE
    REAL(KIND=4), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)         :: msgtag
    CHARACTER(*), INTENT(IN)    :: group
    INTEGER, INTENT(IN)         :: rootginst
    INTEGER, INTENT(OUT)        :: info

    ! 4 == REAL4
    CALL pvmfreduce(func, data, 1, 4, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_real

  SUBROUTINE pvm_reduce_real_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)         :: datatype ! redundant 
         REAL(KIND=4), INTENT(INOUT) :: reduction(:)
         REAL(KIND=4), INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)         :: count
         INTEGER, INTENT(OUT)        :: info
       END SUBROUTINE func
    END INTERFACE
    REAL(KIND=4), INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)         :: msgtag
    CHARACTER(*), INTENT(IN)    :: group
    INTEGER, INTENT(IN)         :: rootginst
    INTEGER, INTENT(OUT)        :: info

    ! 4 == REAL4
    CALL pvmfreduce(func, data, size(data, 1), 4, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_real_array

  SUBROUTINE pvm_reduce_longreal(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)         :: datatype ! redundant 
         REAL(KIND=8), INTENT(INOUT) :: reduction
         REAL(KIND=8), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)         :: count
         INTEGER, INTENT(OUT)        :: info
       END SUBROUTINE func
    END INTERFACE
    REAL(KIND=8), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)         :: msgtag
    CHARACTER(*), INTENT(IN)    :: group
    INTEGER, INTENT(IN)         :: rootginst
    INTEGER, INTENT(OUT)        :: info

    ! 6 == REAL8
    CALL pvmfreduce(func, data, 1, 6, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_longreal

  SUBROUTINE pvm_reduce_longreal_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)         :: datatype ! redundant 
         REAL(KIND=8), INTENT(INOUT) :: reduction(:)
         REAL(KIND=8), INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)         :: count
         INTEGER, INTENT(OUT)        :: info
       END SUBROUTINE func
    END INTERFACE
    REAL(KIND=8), INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)         :: msgtag
    CHARACTER(*), INTENT(IN)    :: group
    INTEGER, INTENT(IN)         :: rootginst
    INTEGER, INTENT(OUT)        :: info

    ! 6 == REAL8
    CALL pvmfreduce(func, data, size(data, 1), 6, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_longreal_array

  SUBROUTINE pvm_reduce_complex(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         COMPLEX(KIND=4), INTENT(INOUT) :: reduction
         COMPLEX(KIND=4), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    COMPLEX(KIND=4), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 5 == COMPLEX8
    CALL pvmfreduce(func, data, 1, 5, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_complex

  SUBROUTINE pvm_reduce_complex_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         COMPLEX(KIND=4), INTENT(INOUT) :: reduction(:)
         COMPLEX(KIND=4), INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    COMPLEX(KIND=4), INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 5 == COMPLEX16
    CALL pvmfreduce(func, data, size(data, 1), 5, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_complex_array

  SUBROUTINE pvm_reduce_longcomplex(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         COMPLEX(KIND=8), INTENT(INOUT) :: reduction
         COMPLEX(KIND=8), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    COMPLEX(KIND=8), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 7 == COMPLEX16
    CALL pvmfreduce(func, data, 1, 7, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_longcomplex

  SUBROUTINE pvm_reduce_longcomplex_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)            :: datatype ! redundant 
         COMPLEX(KIND=8), INTENT(INOUT) :: reduction(:)
         COMPLEX(KIND=8), INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)            :: count
         INTEGER, INTENT(OUT)           :: info
       END SUBROUTINE func
    END INTERFACE
    COMPLEX(KIND=8), INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)            :: msgtag
    CHARACTER(*), INTENT(IN)       :: group
    INTEGER, INTENT(IN)            :: rootginst
    INTEGER, INTENT(OUT)           :: info

    ! 7 == COMPLEX16
    CALL pvmfreduce(func, data, size(data, 1), 7, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_longcomplex_array

  SUBROUTINE pvm_reduce_logical(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)    :: datatype ! redundant 
         LOGICAL, INTENT(INOUT) :: reduction
         LOGICAL, INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)    :: count
         INTEGER, INTENT(OUT)   :: info
       END SUBROUTINE func
    END INTERFACE
    LOGICAL, INTENT(INOUT)   :: data
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! 3 == INTEGER4
    CALL pvmfreduce(func, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_logical

  SUBROUTINE pvm_reduce_logical_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)    :: datatype ! redundant 
         LOGICAL, INTENT(INOUT) :: reduction(:)
         LOGICAL, INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)    :: count
         INTEGER, INTENT(OUT)   :: info
       END SUBROUTINE func
    END INTERFACE
    LOGICAL, INTENT(INOUT)   :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! 3 == INTEGER4
    CALL pvmfreduce(func, data, size(data, 1), 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_logical_array

  SUBROUTINE pvm_reduce_string(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)         :: datatype ! redundant 
         CHARACTER(*), INTENT(INOUT) :: reduction
         CHARACTER(*), INTENT(IN)    :: new_comer
         INTEGER, INTENT(IN)         :: count
         INTEGER, INTENT(OUT)        :: info
       END SUBROUTINE func
    END INTERFACE
    CHARACTER(*), INTENT(INOUT) :: data
    INTEGER, INTENT(IN)         :: msgtag
    CHARACTER(*), INTENT(IN)    :: group
    INTEGER, INTENT(IN)         :: rootginst
    INTEGER, INTENT(OUT)        :: info

    ! 0 == STRING
    CALL pvmfreduce(func, data, len(data), 0, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_string

  SUBROUTINE pvm_reduce_character_array(func, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTERFACE
       SUBROUTINE func (datatype, reduction, new_comer, count, info)
         IMPLICIT NONE

         INTEGER, INTENT(IN)      :: datatype ! redundant 
         CHARACTER, INTENT(INOUT) :: reduction(:)
         CHARACTER, INTENT(IN)    :: new_comer(:)
         INTEGER, INTENT(IN)      :: count
         INTEGER, INTENT(OUT)     :: info
       END SUBROUTINE func
    END INTERFACE
    CHARACTER, INTENT(INOUT) :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! 0 == STRING
    CALL pvmfreduce(func, data, size(data, 1), 0, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_reduce_character_array


  SUBROUTINE pvm_scatter_root_shortinteger(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(OUT) :: result
    INTEGER(KIND=1), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! BYTE1 == 1
    CALL pvmfscatter(result, data, 1, 1, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_shortinteger

  SUBROUTINE pvm_scatter_r_shortint_array(result, data, msgtag, group, &
                                          rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(OUT) :: result(:)
    INTEGER(KIND=1), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! BYTE1 == 1
    CALL pvmfscatter(result, data, size(result,1), 1, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_r_shortint_array

  SUBROUTINE pvm_scatter_shortinteger(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(OUT) :: result
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! BYTE1 == 1
    CALL pvmfscatter(result, 0, 1, 1, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_shortinteger

  SUBROUTINE pvm_scatter_shortinteger_array(result, msgtag, group, &
                                            rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(OUT) :: result(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! BYTE1 == 1
    CALL pvmfscatter(result, 0, size(result,1), 1, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_shortinteger_array

  SUBROUTINE pvm_scatter_root_integer(result, data, msgtag, group, &
                                      rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(OUT) :: result
    INTEGER(KIND=2), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER2 == 2
    CALL pvmfscatter(result, data, 1, 2, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_integer

  SUBROUTINE pvm_scatter_root_integer_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(OUT) :: result(:)
    INTEGER(KIND=2), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER2 == 2
    CALL pvmfscatter(result, data, size(result,1), 2, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_scatter_root_integer_array

  SUBROUTINE pvm_scatter_integer(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(OUT) :: result
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER2 == 2
    CALL pvmfscatter(result, 0, 1, 2, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_integer

  SUBROUTINE pvm_scatter_integer_array(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(OUT) :: result(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER2 == 2
    CALL pvmfscatter(result, 0, size(result,1), 2, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_integer_array

  SUBROUTINE pvm_scatter_root_longinteger(result, data, msgtag, group, &
                                         rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(OUT) :: result
    INTEGER(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER4 == 3
    CALL pvmfscatter(result, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_longinteger

  SUBROUTINE pvm_scatter_r_longint_array(result, data, msgtag, group, &
                                         rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(OUT) :: result(:)
    INTEGER(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! INTEGER4 == 3
    CALL pvmfscatter(result, data, size(result,1), 3, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_scatter_r_longint_array

  SUBROUTINE pvm_scatter_longinteger(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(OUT) :: result
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfscatter(result, 0, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_longinteger

  SUBROUTINE pvm_scatter_longinteger_array(result, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(OUT) :: result(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfscatter(result, 0, size(result,1), 3, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_longinteger_array

  SUBROUTINE pvm_scatter_root_real(result, data, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(OUT) :: result
    REAL(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL4 == 4
    CALL pvmfscatter(result, data, 1, 4, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_real

  SUBROUTINE pvm_scatter_root_real_array(result, data, msgtag, group, &
                                         rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(OUT) :: result(:)
    REAL(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL4 == 4
    CALL pvmfscatter(result, data, size(result,1), 4, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_root_real_array

  SUBROUTINE pvm_scatter_real(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(OUT) :: result
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL4 == 4
    CALL pvmfscatter(result, 0, 1, 4, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_real

  SUBROUTINE pvm_scatter_real_array(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(OUT) :: result(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL4 == 4
    CALL pvmfscatter(result, 0, size(result,1), 4, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_scatter_real_array

  SUBROUTINE pvm_scatter_root_longreal(result, data, msgtag, group, &
                                       rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(OUT) :: result
    REAL(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL8 == 6
    CALL pvmfscatter(result, data, 1, 6, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_longreal

  SUBROUTINE pvm_scatter_root_longreal_array(result, data, msgtag, group, &
                                            rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(OUT) :: result(:)
    REAL(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! REAL8 == 6
    CALL pvmfscatter(result, data, size(result,1), 6, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_scatter_root_longreal_array

  SUBROUTINE pvm_scatter_longreal(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(OUT) :: result
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL8 == 6
    CALL pvmfscatter(result, 0, 1, 6, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_longreal

  SUBROUTINE pvm_scatter_longreal_array(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(OUT) :: result(:)
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! REAL8 == 6
    CALL pvmfscatter(result, 0, size(result,1), 6, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_longreal_array

  SUBROUTINE pvm_scatter_root_complex(result, data, msgtag, group, &
                                      rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(OUT) :: result
    COMPLEX(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX8 = 5
    CALL pvmfscatter(result, data, 1, 5, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_complex

  SUBROUTINE pvm_scatter_root_complex_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(OUT) :: result(:)
    COMPLEX(KIND=4), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX8 = 5
    CALL pvmfscatter(result, data, size(result,1), 5, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_scatter_root_complex_array

  SUBROUTINE pvm_scatter_complex(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(OUT) :: result
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX8 = 5
    CALL pvmfscatter(result, 0, 1, 5, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_complex

  SUBROUTINE pvm_scatter_complex_array(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(OUT) :: result(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX8 = 5
    CALL pvmfscatter(result, 0, size(result,1), 5, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_complex_array

  SUBROUTINE pvm_scatter_root_longcomplex(result, data, msgtag, group, &
                                          rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(OUT) :: result
    COMPLEX(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX16 == 7
    CALL pvmfscatter(result, data, 1, 7, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_longcomplex

  SUBROUTINE pvm_scatter_r_longcmplx_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(OUT) :: result(:)
    COMPLEX(KIND=8), INTENT(IN)  :: data(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    ! COMPLEX16 == 7
    CALL pvmfscatter(result, data, size(result, 1), 7, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_r_longcmplx_array

  SUBROUTINE pvm_scatter_longcomplex(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(OUT) :: result
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX16 == 7
    CALL pvmfscatter(result, 0, 1, 7, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_longcomplex

  SUBROUTINE pvm_scatter_longcomplex_array(result, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(OUT) :: result(:)
    INTEGER, INTENT(IN)          :: msgtag
    CHARACTER(*), INTENT(IN)     :: group
    INTEGER, INTENT(IN)          :: rootginst
    INTEGER, INTENT(OUT)         :: info

    INTEGER(KIND=1) :: ignored
    ! COMPLEX16 == 7
    CALL pvmfscatter(result, 0, size(result,1), 7, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_longcomplex_array

  SUBROUTINE pvm_scatter_root_logical(result, data, msgtag, group, &
                                      rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT)     :: result
    LOGICAL, INTENT(IN)      :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! INTEGER4 == 3
    CALL pvmfscatter(result, data, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_root_logical

  SUBROUTINE pvm_scatter_root_logical_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT)     :: result(:)
    LOGICAL, INTENT(IN)      :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! INTEGER4 == 3
    CALL pvmfscatter(result, data, size(result,1), 3, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_root_logical_array

  SUBROUTINE pvm_scatter_logical(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT)     :: result
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfscatter(result, 0, 1, 3, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_logical

  SUBROUTINE pvm_scatter_logical_array(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    LOGICAL, INTENT(OUT)     :: result(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    INTEGER(KIND=1) :: ignored
    ! INTEGER4 == 3
    CALL pvmfscatter(result, 0, size(result,1), 3, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_scatter_logical_array

  SUBROUTINE pvm_scatter_root_string(result, data, msgtag, group, &
                                     rootginst, info)
    IMPLICIT NONE
    CHARACTER(*), INTENT(OUT) :: result
    CHARACTER(*), INTENT(IN)  :: data
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    ! STRING == 0
    CALL pvmfscatter(result, data, len(data), 0, msgtag, group, &
                    rootginst, info)
  END SUBROUTINE pvm_scatter_root_string

  SUBROUTINE pvm_scatter_string(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    CHARACTER(*), INTENT(OUT) :: result
    INTEGER, INTENT(IN)       :: msgtag
    CHARACTER(*), INTENT(IN)  :: group
    INTEGER, INTENT(IN)       :: rootginst
    INTEGER, INTENT(OUT)      :: info

    INTEGER(KIND=1) :: ignored
    ! STRING == 0
    CALL pvmfscatter(result, 0, len(result), 0, msgtag, group, rootginst, info)
  END SUBROUTINE pvm_scatter_string

  SUBROUTINE pvm_scatter_r_character_array(result, data, msgtag, group, &
                                           rootginst, info)
    IMPLICIT NONE
    CHARACTER, INTENT(OUT)   :: result(:)
    CHARACTER, INTENT(IN)    :: data(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    ! STRING == 0
    CALL pvmfscatter(result, data, size(result,1), 0, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_r_character_array

  SUBROUTINE pvm_scatter_character_array(result, msgtag, group, rootginst, info)
    IMPLICIT NONE
    CHARACTER, INTENT(OUT)   :: result(:)
    INTEGER, INTENT(IN)      :: msgtag
    CHARACTER(*), INTENT(IN) :: group
    INTEGER, INTENT(IN)      :: rootginst
    INTEGER, INTENT(OUT)     :: info

    INTEGER(KIND=1) :: ignored
    ! STRING == 0
    CALL pvmfscatter(result, 0, size(result,1), 0, msgtag, group, &
                     rootginst, info)
  END SUBROUTINE pvm_scatter_character_array


  SUBROUTINE pvm_spawn_one(task, flag, where, tid, info)
    IMPLICIT NONE
    CHARACTER(*), INTENT(IN)  :: task
    INTEGER, INTENT(IN)       :: flag
    CHARACTER(*), INTENT(IN)  :: where ! host or architecture
    INTEGER, INTENT(OUT)      :: tid
    INTEGER, INTENT(OUT)      :: info

    ! local declaration
    INTEGER :: numt

    CALL pvmfspawn(task, flag, where, 1, tid, numt)
    IF (numt == 1) THEN
       info = PvmOK
    ELSE
       info = tid
    END IF
  END SUBROUTINE pvm_spawn_one

  SUBROUTINE pvm_spawn_many(task, flag, where, ntask, tids, numt)
    IMPLICIT NONE
    CHARACTER(*), INTENT(IN) :: task
    INTEGER, INTENT(IN)      :: flag
    CHARACTER(*), INTENT(IN) :: where ! host or architecture
    INTEGER, INTENT(IN)      :: ntask
    INTEGER, INTENT(OUT)     :: tids(ntask)
    INTEGER, INTENT(OUT)     :: numt

    CALL pvmfspawn(task, flag, where, ntask, tids, numt)
  END SUBROUTINE pvm_spawn_many


  SUBROUTINE pvm_unpack_shortinteger(data, info)
    IMPLICIT NONE
    INTEGER(KIND=1), POINTER :: data
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfunpack(1, data, 1, 1, info) ! 1 == BYTE1
  END SUBROUTINE pvm_unpack_shortinteger

  SUBROUTINE pvm_unpack_shortinteger_array(data, count, stride, info)
    IMPLICIT NONE
    INTEGER(KIND=1), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(1, data, count, stride, info) ! 1 == BYTE1
  END SUBROUTINE pvm_unpack_shortinteger_array

  SUBROUTINE pvm_unpack_integer(data, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(2, data, 1, 1, info) ! 2 == INTEGER2
  END SUBROUTINE pvm_unpack_integer

  SUBROUTINE pvm_unpack_integer_array(data, count, stride, info)
    IMPLICIT NONE
    INTEGER(KIND=2), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(2, data, count, stride, info) ! 2 == INTEGER2
  END SUBROUTINE pvm_unpack_integer_array

  SUBROUTINE pvm_unpack_longinteger(data, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(3, data, 1, 1, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_unpack_longinteger

  SUBROUTINE pvm_unpack_longinteger_array(data, count, stride, info)
    IMPLICIT NONE
    INTEGER(KIND=4), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(3, data, count, stride, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_unpack_longinteger_array

  SUBROUTINE pvm_unpack_real(data, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(IN) :: data
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfunpack(4, data, 1, 1, info) ! 4 == REAL4
  END SUBROUTINE pvm_unpack_real

  SUBROUTINE pvm_unpack_real_array(data, count, stride, info)
    IMPLICIT NONE
    REAL(KIND=4), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)      :: count
    INTEGER, INTENT(IN)      :: stride
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfunpack(4, data, count, stride, info) ! 4 == REAL4
  END SUBROUTINE pvm_unpack_real_array

  SUBROUTINE pvm_unpack_longreal(data, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN) :: data
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfunpack(6, data, 1, 1, info) ! 6 == REAL8
  END SUBROUTINE pvm_unpack_longreal

  SUBROUTINE pvm_unpack_longreal_array(data, count, stride, info)
    IMPLICIT NONE
    REAL(KIND=8), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)      :: count
    INTEGER, INTENT(IN)      :: stride
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfunpack(6, data, count, stride, info) ! 6 == REAL8
  END SUBROUTINE pvm_unpack_longreal_array

  SUBROUTINE pvm_unpack_complex(data, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(5, data, 1, 1, info) ! 5 == COMPLEX8
  END SUBROUTINE pvm_unpack_complex

  SUBROUTINE pvm_unpack_complex_array(data, count, stride, info)
    IMPLICIT NONE
    COMPLEX(KIND=4), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(5, data, count, stride, info) ! 5 == COMPLEX8
  END SUBROUTINE pvm_unpack_complex_array

  SUBROUTINE pvm_unpack_longcomplex(data, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(IN) :: data
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(7, data, 1, 1, info) ! 7 == COMPLEX16
  END SUBROUTINE pvm_unpack_longcomplex

  SUBROUTINE pvm_unpack_longcomplex_array(data, count, stride, info)
    IMPLICIT NONE
    COMPLEX(KIND=8), INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)         :: count
    INTEGER, INTENT(IN)         :: stride
    INTEGER, INTENT(OUT)        :: info

    CALL pvmfunpack(7, data, count, stride, info) ! 7 == COMPLEX16
  END SUBROUTINE pvm_unpack_longcomplex_array

  SUBROUTINE pvm_unpack_logical(data, info)
    IMPLICIT NONE
    LOGICAL, POINTER      :: data
    INTEGER, INTENT(OUT)  :: info

    CALL pvmfunpack(3, data, 1, 1, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_unpack_logical

  SUBROUTINE pvm_unpack_logical_array(data, count, stride, info)
    IMPLICIT NONE
    LOGICAL, INTENT(IN)   :: data(:)
    INTEGER, INTENT(IN)   :: count
    INTEGER, INTENT(IN)   :: stride
    INTEGER, INTENT(OUT)  :: info

    CALL pvmfunpack(3, data, count, stride, info) ! 3 == INTEGER4
  END SUBROUTINE pvm_unpack_logical_array

  SUBROUTINE pvm_unpack_string(data, info)
    IMPLICIT NONE
    CHARACTER(*), INTENT(IN) :: data
    INTEGER, INTENT(OUT)     :: info

    CALL pvmfunpack(0, data, LEN(data), 1, info) ! 0 == STRING
  END SUBROUTINE pvm_unpack_string

  SUBROUTINE pvm_unpack_character_array(data, count, stride, info)
    IMPLICIT NONE
    CHARACTER, INTENT(IN) :: data(:)
    INTEGER, INTENT(IN)   :: count
    INTEGER, INTENT(IN)   :: stride
    INTEGER, INTENT(OUT)  :: info

    CALL pvmfunpack(0, data, size(data, 1), 1, info) ! 0 == STRING
  END SUBROUTINE pvm_unpack_character_array

END MODULE PVM_Interfaces
